function im_bw = apply_bw_correction(im, low, high, gamma)
%*************************************************************************%
% Author: Dominic Rfenacht, April 2012
% Purpose:
% This function is used to perform the black/whitepoint correction based on
% quantiles, as well as a global tonemapping. If the low and high parameters are not specified, default
% values are used (low = 0.005, high = 0.995), and gamma is set to 1.0 (linear).
%*************************************************************************%
%Input: - im: Input image
%       - low: value of the low quantile (default: 0.005)
%       - high: Value of the high quantile (default: 0.995)
%       - gamma: Value for gamma correction (default: 1.0)
%Output: - im_bw: Black/Whitepoint and gamma corrected image
%*************************************************************************%

if(nargin == 1)
    low = 0.005;
    high = 0.995;
    gamma = 1.0;
elseif(nargin ~= 4)
    error('Please use either 4 or 1 argument');
end

if(size(im, 3) == 3)
    %Convert to YCbCr
    im = rgb2ycbcr(im);
    %Extract luminance channel
    Y = im(:,:,1);
    
    %Find lower and upper quantile
    low_high = stretchlim(Y, [low high]);
    
    %Perform B/W and global gamma correction
    Y = imadjust(Y, [low_high(1); low_high(2)], [16/255; 235/255], gamma); % see matlab help of rgb2ycbcr to know why 16/255 and 235/255
    im(:,:,1) = Y;
    
    %Convert back to RGB
    im_bw = ycbcr2rgb(im);
    
elseif(size(im,3) == 1)
    %Find lower and upper quantile
    low_high = stretchlim(im, [low high]);
    
    %Perform B/W and global gamma correction
    im_bw = imadjust(im, [low_high(1); low_high(2)], [], gamma);
else
    error('Image must either have three or one color channels!');
end

