function xout = apply_demosaicing(handle, type)


switch type
    case 'vis'
        xin = handle.xv;
    case 'nir'
        xin = handle.xn;
        spectral_sensitivities = handle.cameras{handle.rowCam2, 4};
    case 'nir_only'
        xin = handle.xn;
        spectral_sensitivities = handle.cameras{handle.rowCam1, 4};
    otherwise
        error('Specified type does not exist!')
end

cfa_pattern = handle.cameras{handle.rowCam1, 3};
nir_info = handle.nir_info;
method = handle.demosaicing_method;

ind = get_cfa_index(cfa_pattern);
red_ind = ind(1,:);
green1_ind = ind(2,:);
green2_ind = ind(4,:);
blue_ind = ind(3,:);

if(~strcmp(nir_info, 'all'))
    nir_ind = ind(getIndexFromString(nir_info), :);
else
    nir_ind = ind(getIndexFromString('R'), :);
end

switch method
    case 'Amaze'
        switch cfa_pattern
            case 'RGGB'
                cfa_type = 1;
            case 'GBRG'
                cfa_type = 2;
            case 'BGGR'
                cfa_type = 3;
            case 'GRBG'
                cfa_type = 4;
            otherwise
                error('CFA type does not exist...')
        end
        
        switch type
            case 'vis'
                try
                    xout = amaze(xin, cfa_type);
                catch
                    disp('Compiling Amaze for your system... This takes a while!')
                    try
                        mex amaze.cc
                        disp('Done...')
                        xout = amaze(xin, cfa_type);
                    catch
                        error('Compiling failed. Make sure that gcc is installed!')
                    end
                    
                end
                
            case {'nir', 'nir_only'}
                xn_mosaic = zeros([size(xin), 3]);
                count = 1;
                for i = 1:2
                    for j = 1:2
                        xn_mosaic(i:2:end, j:2:end, getIndexFromString(cfa_pattern(count))) = xin(i:2:end, j:2:end);
                        count = count + 1;
                    end
                end
                try
                    xout = amaze(xn_mosaic, cfa_type);
                catch
                    disp('Compiling Amaze for your system...')
                    try
                        mex amaze.cc
                        disp('Done...')
                        xout = amaze(xn_mosaic, cfa_type);
                    catch
                        error('Compiling failed. Make sure that gcc is installed!')
                    end
                end
                    
                xout = rgb2gray(xout./max(max(xout(:))));
        end
        
        
        %Bilinear interpolation -> Fast, but generally poor results, especially
        %for low-resolution images
    case 'Bilinear interpolation'
        switch type
            case 'vis'
                %Apply bilinear interpolation
                xout = interp_bilinear(xin, 'RGB');
            case {'nir', 'nir_only'}
                
                %If the nir information is taken from all three color channels, we
                %only reweight the intensities according to the quantum
                %efficiencies. Otherwise, normal interpolation is used.
                if(strcmp(nir_info, 'all'))
                    xout = zeros(size(xin));
                    spectrum = 38:length(spectral_sensitivities);
                    quant_eff = [trapz(spectral_sensitivities(spectrum,1), spectral_sensitivities(spectrum,2)), trapz(spectral_sensitivities(spectrum,1), spectral_sensitivities(spectrum,3)), trapz(spectral_sensitivities(spectrum,1), spectral_sensitivities(spectrum,4))];
                    new_weights = quant_eff(1)./quant_eff;
                    
                    xout(red_ind(1):2:end, red_ind(2):2:end) = xin(red_ind(1):2:end, red_ind(2):2:end);
                    xout(green1_ind(1):2:end, green1_ind(2):2:end) = new_weights(2).*xin(green1_ind(1):2:end, green1_ind(2):2:end);
                    xout(green2_ind(1):2:end, green2_ind(2):2:end) = new_weights(2).*xin(green2_ind(1):2:end, green2_ind(2):2:end);
                    xout(blue_ind(1):2:end, blue_ind(2):2:end) = new_weights(3).*xin(blue_ind(1):2:end, blue_ind(2):2:end);
                    
                    
                else
                    xout = interp_bilinear(xin, nir_info);
                end
        end
        
    case {'Subsample by 2', 'Subsample by 4', 'Subsample by 8'}
        subsamp_fac = str2num(method(end));
        [m,n,~] = size(xin);
        cut1 = mod(m, subsamp_fac);
        cut2 = mod(n, subsamp_fac);
        
        switch type
            case 'vis'
                xout = zeros(floor(m/subsamp_fac), floor(n/subsamp_fac), 3);
                
                ind = get_cfa_index(cfa_pattern);
                red_ind = ind(1,:);
                green1_ind = ind(2,:);
                green2_ind = ind(4,:);
                blue_ind = ind(3,:);
                
                red = xin(red_ind(1):2:end-cut1, red_ind(2):2:end-cut2, 1);
                green = (xin(green1_ind(1):2:end-cut1, green1_ind(2):2:end-cut2, 2) + xin(green2_ind(1):2:end-cut1, green2_ind(2):2:end-cut2, 2))./2;
                blue = xin(blue_ind(1):2:end-cut1, blue_ind(2):2:end-cut2, 3);
                
            case {'nir', 'nir_only'}
                nir = xin(nir_ind(1):2:end-cut1, nir_ind(2):2:end-cut2, getIndexFromString(nir_info));
        end
        
        step = subsamp_fac/2;
        %Filter to avoid aliasing
        %Get sigma for Gaussian filter
        sigma = 0.289*subsamp_fac;
        h = gaussFilter(sigma);
        switch type
            case 'vis'
                %Subsample
                red = imfilter(red, h, 'symmetric','same','conv');
                green = imfilter(green, h, 'symmetric','same','conv');
                blue = imfilter(blue, h, 'symmetric','same','conv');
                xout(:,:,1) = red(1:step:end, 1:step:end);
                xout(:,:,2) = green(1:step:end, 1:step:end);
                xout(:,:,3) = blue(1:step:end, 1:step:end);
            case {'nir', 'nir_only'}
                nir = imfilter(nir, h, 'symmetric','same','conv');
                xout = nir(1:step:end, 1:step:end);
        end
        
        %{
            %%NEEDS TO BE ADAPTED, BUT ANYWAYS WORSE THAN AMAZE
                %Alternating projections, as described in the paper:
                %	Color plane interpolation using alternating projections
                %   Gunturk, B.K.; Altunbasak, Y.; Mersereau, R.M.
                %	Image Processing, IEEE Transactions on , Volume: 11 Issue: 9 , Sept 2002
                %	Page(s): 997 -1013
            case 'Alternating projections (AP)'
                %Apply bilinear interpolation for initialization
                xv = interp_bilinear(xv, 'RGB');
                xn = interp_bilinear(xn, nir_info);
                
                %Threshold
                T = 0.01;
                %Number of iterations
                N_iter = 2;
                
                %%%%% Initial interpolation of the color channels
                %RGB = double(interp_edgeG(X));
                RGB = esi_rgb(xv, cfa_pattern);
                %%%%% Initial Red, green, and blue channel assignments
                R = RGB(:,:,1);
                G = RGB(:,:,2);
                B = RGB(:,:,3);
                N = xn;
                %%%%% Filters that will be used in subband decomposition
                h0 = [1 2 1]/4;
                h1 = [1 -2 1]/4;
                g0 = [-1 2 6 2 -1]/8;
                g1 = [1 2 -6 2 1]/8;
                
                %%%% Update Green channel
                Rd = xv(red_ind(1):2:end, red_ind(2):2:end, 1);
                Bd = xv(blue_ind(1):2:end,blue_ind(2):2:end, 3);
                Gd_R = RGB(red_ind(1):2:end,red_ind(2):2:end,2);
                Gd_B = RGB(blue_ind(1):2:end,blue_ind(2):2:end,2);
                %
                [CA_Rr,CH_Rr,CV_Rr,CD_Rr] = rdwt2(Rd,h0,h1);
                [CA_Gr,CH_Gr,CV_Gr,CD_Gr] = rdwt2(Gd_R,h0,h1);
                [CA_Bb,CH_Bb,CV_Bb,CD_Bb] = rdwt2(Bd,h0,h1);
                [CA_Gb,CH_Gb,CV_Gb,CD_Gb] = rdwt2(Gd_B,h0,h1);
                %
                Gd_R = ridwt2(CA_Gr, CH_Rr, CV_Rr, CD_Rr, g0,g1);
                Gd_B = ridwt2(CA_Gb, CH_Bb, CV_Bb, CD_Bb, g0,g1);
                %
                G(red_ind(1):2:end,red_ind(2):2:end) = Gd_R;
                G(blue_ind(1):2:end,blue_ind(2):2:end) = Gd_B;
                
                
                %%%%% Alternating projections algorithm starts here
                for iter=1:N_iter, % iterate
                    %%%%% DETAIL PROJECTION
                    %%%%% Decompose into subbands
                    [CA_R,CH_R,CV_R,CD_R] = rdwt2(R,h0,h1);
                    [CA_G,CH_G,CV_G,CD_G] = rdwt2(G,h0,h1);
                    [CA_B,CH_B,CV_B,CD_B] = rdwt2(B,h0,h1);
                    
                    [CA_N,CH_N,CV_N,CD_N] = rdwt2(N,h0,h1);
                    
                    %%%%% Update detail subbands of R, B and N
                    %%% R
                    CH_R(CH_R-CH_G > T) = CH_G(CH_R-CH_G > T) + T;
                    CH_R(CH_R-CH_G <-T) = CH_G(CH_R-CH_G <-T) - T;
                    CV_R(CV_R-CV_G > T) = CV_G(CV_R-CV_G > T) + T;
                    CV_R(CV_R-CV_G <-T) = CV_G(CV_R-CV_G <-T) - T;
                    CD_R(CD_R-CD_G > T) = CD_G(CD_R-CD_G > T) + T;
                    CD_R(CD_R-CD_G <-T) = CD_G(CD_R-CD_G <-T) - T;
                    %%% B
                    CH_B(CH_B-CH_G > T) = CH_G(CH_B-CH_G > T) + T;
                    CH_B(CH_B-CH_G <-T) = CH_G(CH_B-CH_G <-T) - T;
                    CV_B(CV_B-CV_G > T) = CV_G(CV_B-CV_G > T) + T;
                    CV_B(CV_B-CV_G <-T) = CV_G(CV_B-CV_G <-T) - T;
                    CD_B(CD_B-CD_G > T) = CD_G(CD_B-CD_G > T) + T;
                    CD_B(CD_B-CD_G <-T) = CD_G(CD_B-CD_G <-T) - T;
                    %%% N
                    CH_N(CH_N-CH_G > T) = CH_G(CH_N-CH_G > T) + T;
                    CH_N(CH_N-CH_G <-T) = CH_G(CH_N-CH_G <-T) - T;
                    CV_N(CV_N-CV_G > T) = CV_G(CV_N-CV_G > T) + T;
                    CV_N(CV_N-CV_G <-T) = CV_G(CV_N-CV_G <-T) - T;
                    CD_N(CD_N-CD_G > T) = CD_G(CD_N-CD_G > T) + T;
                    CD_N(CD_N-CD_G <-T) = CD_G(CD_N-CD_G <-T) - T;
                    
                    %%%%% Reconstruct back from subbands
                    R = ridwt2(CA_R, CH_R, CV_R, CD_R, g0, g1);
                    B = ridwt2(CA_B, CH_B, CV_B, CD_B, g0, g1);
                    N = ridwt2(CA_N, CH_N, CV_N, CD_N, g0, g1);
                    
                    %%%%% OBSERVATION PROJECTION
                    %%%%% Make sure that R and B channels obey the data
                    R(red_ind(1):2:end, red_ind(2):2:end) = xv(red_ind(1):2:end, red_ind(2):2:end, 1);
                    B(blue_ind(1):2:end, blue_ind(2):2:end) = xv(blue_ind(1):2:end, blue_ind(2):2:end, 3);
                    N(nir_ind(1):2:end, nir_ind(2):2:end) = xn(nir_ind(1):2:end, nir_ind(2):2:end);
                    
                end
                xv_out = zeros(size(xv));
                %%%%% Output the image...
                xv_out(:,:,1) = R;
                xv_out(:,:,2) = G;
                xv_out(:,:,3) = B;
                
                xn_out = N;
        %}
        
    otherwise
        error('Demosaicing method does not exist. Feel free to add it ;-)')
end