function im_out = apply_white_balancing(im_in, im_nir, mode, cfa)
%*************************************************************************%
% Author: Dominic Rfenacht
% Last revision: 04.05.2012
% Purpose:
% This function is used to apply white balancing to the image.
%*************************************************************************%
%Input: - im_in: Input image
%       - mode: Type of white balancing algorithm used. 

%Output: - im_out: White-balanced image
%*************************************************************************%

if(nargin == 4)
    ind = get_cfa_index(cfa);
    red = im_in(ind(1,1):2:end, ind(1,2):2:end, 1);
    green = im_in(ind(2,1):2:end, ind(2,2):2:end, 2);
    blue = im_in(ind(3,1):2:end, ind(3,2):2:end, 3);
    
elseif(nargin == 3)
    red = im_in(:,:,1);
    green = im_in(:,:,2);
    blue = im_in(:,:,3);
else
    error('Wrong number of arguments.');
end


[m,n] = size(red);

switch mode
    case 'None'
        im_out = im_in;
    
    case 'Modified Grey World'
        %There need to be at least 0.1% of the pixels to have this value so
        %that its bin is counted
        T = m*n*0.001;
        [count_red, bin_red] = imhist(red./max(max(im_in(:))), 256);
        [count_green, bin_green] = imhist(green./max(max(im_in(:))), 256);
        [count_blue, bin_blue] = imhist(blue./max(max(im_in(:))), 256);
        red_take_bins = bin_red(count_red > T);
        green_take_bins = bin_green(count_green > T);
        blue_take_bins = bin_blue(count_blue > T);
              
        avgR = mean(mean(red_take_bins));
        avgG = mean(mean(green_take_bins));
        avgB = mean(mean(blue_take_bins));
        avgGray = (avgR + avgG + avgB)/3;
               
        im_out = zeros(size(im_in));

        % Average R, G, B value
        if(avgR == 0)
            im_out(:,:,1) = im_in(:,:,1);
        else
            im_out(:,:,1) = (avgGray/avgR).*im_in(:,:,1);
        end
        
        if(avgG == 0)
            im_out(:,:,2) = im_in(:,:,2);
        else
            im_out(:,:,2) = (avgGray/avgG).*im_in(:,:,2);
        end
        
        if(avgB == 0)
            im_out(:,:,3) = im_in(:,:,3);
        else
            im_out(:,:,3) = (avgGray/avgB).*im_in(:,:,3);
        end
        
        
    case {'grey_world', 'Grey World'}
        
        avgR = mean(mean(red));
        avgG = mean(mean(green));
        avgB = mean(mean(blue));
        avgGray = (avgR + avgG + avgB)/3.0;

        
        im_out = zeros(size(im_in));
        %Average R, G, B value
        if(avgR == 0)
            im_out(:,:,1) = im_in(:,:,1);
        else
            im_out(:,:,1) = (avgGray/avgR).*im_in(:,:,1);
        end
        
        if(avgG == 0)
            im_out(:,:,2) = im_in(:,:,2);
        else
            im_out(:,:,2) = (avgGray/avgG).*im_in(:,:,2);
        end
        
        if(avgB == 0)
            im_out(:,:,3) = im_in(:,:,3);
        else
            im_out(:,:,3) = (avgGray/avgB).*im_in(:,:,3);
        end
        
    case {'max_rgb', 'MaxRGB'}
        %Blur the image to get rid of outliers before finding the max...
        sigma = 5;
        
        toto = zeros(m, n, 3);
        f = gaussFilter(sigma);
        toto(:,:,1) = imfilter(red, f, 'symmetric', 'same', 'conv');
        toto(:,:,2) = imfilter(green, f, 'symmetric', 'same', 'conv');
        toto(:,:,3) = imfilter(blue, f, 'symmetric', 'same', 'conv');
        
        %Compute the max in the blurred image
        maxR = max(max(toto(:,:,1)));
        maxG = max(max(toto(:,:,2)));
        maxB = max(max(toto(:,:,3)));
        maxRGB = max([maxR, maxG, maxB]);
        
        im_out = zeros(size(im_in));
        
        if(maxR == 0)
            im_out(:,:,1) = im_in(:,:,1);
        else
            im_out(:,:,1) = (maxRGB/maxR).*im_in(:,:,1);
        end
        
        if(maxG == 0)
            im_out(:,:,2) = im_in(:,:,2);
        else
            im_out(:,:,2) = (maxRGB/maxG).*im_in(:,:,2);
        end
        
        if(maxB == 0)
            im_out(:,:,3) = im_in(:,:,3);
        else
            im_out(:,:,3) = (maxRGB/maxB).*im_in(:,:,3);
        end
    case 'NIR WB'
        if(isempty(im_nir))
            error('NIR image needed for this type of white balancing!');
        end
        
        perc = 0.001;
        V_tot=(double(red).^2+double(green).^2+double(blue).^2);
        toto=flipud(sort(V_tot(:)));
        cutoff = round(perc*(m/4)*(n/4));
        new_val = toto(cutoff);
        F = find(V_tot>=new_val);
        
        ratio_red = mean(double(red(F)))/mean(double(im_nir(F)));
        ratio_green = mean(double(green(F)))/mean(double(im_nir(F)));
        %mean(double(G2(F)))/mean(double(NIR(F)))
        ratio_blue = mean(double(blue(F)))/mean(double(im_nir(F)));
        
        im_out(:,:,1) = ratio_red.*im_in(:,:,1);
        im_out(:,:,2) = ratio_green.*im_in(:,:,2);
        im_out(:,:,3) = ratio_blue.*im_in(:,:,3);
        
                
    otherwise
        error('Specified white balancing method is not implemented');
end