#include "helpers.h"

int ConsistentH(const CMatrixf &u1h, const CMatrixf &u2h, const CMatrixf H, const CMatrixf Hinv, const CArrayi &imageSize1, const CArrayi &imageSize2, float epsilon, CArrayi &consistent, CMatrixf &dist, CMatrixf &u1ht, CMatrixf &u2ht)
{
    int nMatches = u1h.nCols();

    int maxDim1 = max(imageSize1[0], imageSize1[1]);
    int maxDim2 = max(imageSize2[0], imageSize2[1]);
    float epsilon1 = epsilon * maxDim1;
    float epsilon2 = epsilon * maxDim2;
    float epsilon1sq = epsilon1 * epsilon1;
    float epsilon2sq = epsilon2 * epsilon2;

    dist = 0.0f;
    consistent = 0;
    int nConsistent = 0;

    // forward project

    Multiply(u1ht, H, u1h);
    NormaliseHomInPlace(u1ht);
    
    /*cerr << "u1h " << u1h.Dump();
      cerr << "u1ht " << u1ht.Dump();
      cerr << "u2h " << u2h.Dump();*/
   
    for (int m = 0; m < nMatches; m++)
    {
        float diff1 = u1ht(0, m) - u2h(0, m);
        dist(1, m) += diff1 * diff1;
        float diff2 = u1ht(1, m) - u2h(1, m);
        dist(1, m) += diff2 * diff2;
    }

    // backward project

    Multiply(u2ht, Hinv, u2h);
    NormaliseHomInPlace(u2ht);
    
    /*cerr << "u2h " << u2h.Dump();
      cerr << "u2ht " << u2ht.Dump();
      cerr << "u1h " << u1h.Dump();    */

    for (int m = 0; m < nMatches; m++)
    {
        float diff1 = u2ht(0, m) - u1h(0, m);
        dist(0, m) += diff1 * diff1;
        float diff2 = u2ht(1, m) - u1h(1, m);
        dist(0, m) += diff2 * diff2;
    }

    //cerr << "dist = " << dist.Dump() << endl;

    // count consistent
    
    for (int m = 0; m < nMatches; m++)
    {
        if ((dist(0, m) < epsilon2sq) && (dist(1, m) < epsilon1sq))
        {
            consistent[m] = 1;
            nConsistent++;
        }
    }

    return(nConsistent);

}// end ConsistentH


void RANSACSimilarity(CArrayi &inliersID, CMatrixf &bestS, const CMatrixf &u1, const CMatrixf &u2, const CArrayi &imSize1, const CArrayi &imSize2, const CParameter &Param)
{
    int nIterations = Param.Geti("nIterations");
    float epsilon = Param.Getf("epsilon");

    int nMatches = u1.nCols();
    CArrayi inliers(nMatches);
    CMatrixf dist(2, nMatches);
    int bestNInliers = 0;
    bestS.Create(3, 3);
    bestS.eye();

    CMatrixf u1r(2, 2);
    CMatrixf u2r(2, 2);
    CArrayi r(2);

    CMatrixf u1h, u2h, u1ht, u2ht;
    Hom(u1h, u1);
    Hom(u2h, u2);
    u1ht.Create(3, u1.nCols());
    u2ht.Create(3, u2.nCols());

    for (int i = 0; i < nIterations; i++)
    {
        r[0] = rand()%nMatches;
        r[1] = rand()%nMatches;
        
        SelectCols(u1r, u1, r);
        SelectCols(u2r, u2, r);

        CMatrixf S = ComputeSimilarity(u1r, u2r);
        CMatrixf Sinv = Inverse3x3(S);
        
        int nInliers = ConsistentH(u1h, u2h, S, Sinv, imSize1, imSize2, epsilon, inliers, dist, u1ht, u2ht);

        if (nInliers > bestNInliers)
        {
            bestNInliers = nInliers;
            bestS = S;
        }
    }

    CMatrixf bestSinv = Inverse3x3(bestS);    
    ConsistentH(u1h, u2h, bestS, bestSinv, imSize1, imSize2, epsilon, inliers, dist, u1ht, u2ht);
    Findi(inliersID, inliers);
}

void RatioTest(CArrayi &matchID, const CMatrixf &NNdist, float outlierDist)
{
    vector<int> matchIDv;
 
    for (int i = 0; i < NNdist.nCols(); i++)
    {
        float d1NN = NNdist(0, i);
        float d2NN = NNdist(1, i);
        
        float r = d1NN/d2NN;
        if (r < outlierDist)
            matchIDv.push_back(i);            
    }
    
    Convert(matchID, matchIDv);
}    
 
void FindNN(CMatrixi &NN, CMatrixf &NNdist, const CMatrixf &desc1, const CMatrixf &desc2)
{
    NN.Create(2, desc1.nCols());
    NNdist.Create(2, desc1.nCols());

    int nDims = desc1.nRows();
    int rowStride1 = desc1.rowStride();
    int rowStride2 = desc2.rowStride();

    for (int i = 0; i < desc1.nCols(); i++)
    {
        float* desc1ptr0 = &desc1.elementReference(0, i);
     
        float dist1NN(FLT_MAX), dist2NN(FLT_MAX);
        int   id1NN(-1), id2NN(-1);

        // find NN
        for (int j = 0; j < desc2.nCols(); j++)
        {
            float* desc2ptr = &desc2.elementReference(0, j);
            float* desc1ptr = desc1ptr0;

            float sumsqdiff = 0.0f;

            for (int k = 0; k < nDims; k++)
            {
                float diff = *desc1ptr - *desc2ptr;
                desc1ptr += rowStride1;
                desc2ptr += rowStride2;

                sumsqdiff += diff * diff;
            }

            float dist = sqrt(sumsqdiff);
  
            if (dist < dist1NN)
            {
                dist2NN = dist1NN;
                id2NN   = id1NN;                
                dist1NN = dist;
                id1NN   = j;
            }
            else if (dist < dist2NN)
            {
                dist2NN = dist;
                id2NN   = j;
            }
                            
        }
        
        NN(0, i) = id1NN;
        NN(1, i) = id2NN;
        NNdist(0, i) = dist1NN;
        NNdist(1, i) = dist2NN;        
    }
} 

CMatrixf Inverse3x3(const CMatrixf &A)
{
    CMatrixf Ainv(3, 3);
    
    float a = A(0, 0); float b = A(0, 1); float c = A(0, 2);
    float d = A(1, 0); float e = A(1, 1); float f = A(1, 2);
    float g = A(2, 0); float h = A(2, 1); float i = A(2, 2);

    Ainv(0, 0) = e * i - f * h;
    Ainv(0, 1) = - (d * i - f * g);
    Ainv(0, 2) = d * h - e * g;
    Ainv(1, 0) = - (b * i - c * h);
    Ainv(1, 1) = a * i - c * g;
    Ainv(1, 2) = - (a * h - b * g);
    Ainv(2, 0) = b * f - c * e;
    Ainv(2, 1) = - (a * f - c * d);
    Ainv(2, 2) = a * e - b * d;
    
    float det = a * e * i + b * f * g + c * d * h - c * e * g - a * f * h - b * d * i;
    
    Ainv /= det;
    Ainv = ~Ainv;

    return Ainv;
}

CMatrixf ComputeSimilarity(const CMatrixf &u0, const CMatrixf &u1)
{
    // u1 = S * u0

    CMatrixf S(3, 3);
    S.eye();

    int nPoints = u0.nCols();
    assert(u1.nCols() == nPoints);
    assert(nPoints >= 2);

    CMatrixf M(nPoints * 2, 4);
    CMatrixf x(nPoints * 2, 1);
    CMatrixf s(4, 1);
    
    for (int i = 0; i < nPoints; i++)
    {
        M(2 * i, 0)     = u0(0, i);
        M(2 * i, 1)     = u0(1, i);
        M(2 * i, 2)     = 1.0f;
        M(2 * i + 1, 0) = u0(1, i);
        M(2 * i + 1, 1) = -u0(0, i);
        M(2 * i + 1, 3) = 1.0f;
        
        x(2 * i, 0)     = u1(0, i);
        x(2 * i + 1, 0) = u1(1, i); 
    }
    
    if (nPoints == 2)
        SolveLinearSystem(s, M, x);
    else
        SolveLeastSquares(s, M, x);    
    
    float a = s[0];
    float b = s[1];
    float c = s[2];
    float d = s[3];

    S(0, 0) = a;  S(0, 1) = b; S(0, 2) = c;
    S(1, 0) = -b; S(1, 1) = a; S(1, 2) = d;
    
    return S;
}

void SolveLinearSystem(CMatrixf &x, const CMatrixf &A0, const CMatrixf &b0)
{
    // Solve the linear system A * x = b for x
    // using Gaussian Elimination with pivoting
    
    assert(A0.nRows() == A0.nCols());
    assert(A0.nCols() == b0.nData());

    CMatrixf A = A0;
    CMatrixf b = b0;

    int n = b.nData();
    x.Create(n, 1);
    
    for (int i = 0; i < n; i++)
    {
        // Swap row with max pivot value
        float maxval = (float)fabs(A(i, i));
        int   maxrow = i;

        for (int row = i + 1; row < n; row++)
        {
            if ( (float)fabs(A(row, i)) > maxval)
            {
                maxval = (float)fabs(A(row, i));
                maxrow = row;
            }
        }

        if (maxrow != i)
        {
            for (int j = i; j < n; j++)
                Swap(A(i, j), A(maxrow, j));
            
            Swap(b[i], b[maxrow]);            
        }

        // Eliminate ith unknown from subsequent equations
        for (int j = i + 1; j < n; j++)
        {
            float scalej = A(j, i) / A(i, i);

            A(j, i) = 0;
            for (int k = i + 1; k < n; k++)
                A(j, k) -= A(i, k) * scalej;           
            b[j] -= b[i] * scalej;
        }
        
    }// end for i
    
    // Back substitution    
    for (int i = n - 1; i >= 0; i--)
    {
        float rhs = b[i];
        for (int j = i + 1; j < n; j++)
            rhs -= A(i, j) * x[j];
        x[i] = rhs / A(i, i);
    }

}

void SolveLeastSquares(CMatrixf &x, const CMatrixf &A, const CMatrixf &b)
{
    assert(A.nRows() == b.nRows());
    
    // TODO: fast compute AtA
    CMatrixf At, AtA;
    Transpose(At, A);
    Multiply(AtA, At, A);
    CMatrixf Atb;
    Multiply(Atb, At, b);
    SolveLinearSystem(x, AtA, Atb);
}

void ComputeImageCoords(CImagef &imCoord, const CArrayi &imSize, const CMatrixf &S)
{
    int nRows = imSize[0];
    int nCols = imSize[1];

    imCoord.Create(nRows, nCols, 2);

    CMatrixf x1(3, 1), x2(3, 1);

    for (int i = 0; i < nRows; i++)
    {
        for (int j = 0; j < nCols; j++)
        {
            x1[0] = i;
            x1[1] = j;
            x1[2] = 1.0f;
            
            Multiply(x2, S, x1);
            
            imCoord(i, j, 0) = x2[0];
            imCoord(i, j, 1) = x2[1];
        }
    }
}

void SampleImageBilinear(CImagef &renderIm, CImagei &alphaIm, const CImagef &imCoord, const CImagef &image)
{
    int renderRows = imCoord.nRows();
    int renderCols = imCoord.nCols();
    int imRows     = image.nRows();
    int imCols     = image.nCols();
    int nBands     = image.nBands();
    int nPixels    = renderRows * renderCols;

    renderIm.Create(renderRows, renderCols, nBands);
    alphaIm.Create(renderRows, renderCols, 1);
    alphaIm = 0;
    
    float* imCoordPtr  = imCoord.m_data;
    int* alphaImPtr = alphaIm.m_data;
    float* renderImPtr = renderIm.m_data;
    int imageRowStride = image.rowStride();
 
    float imRows2 = (imRows - 1.0f)/2.0f;
    float imCols2 = (imCols - 1.0f)/2.0f;
    float imRowsCols2 = imRows2 * imCols2;

    for (int i = 0; i < nPixels; i++)
    {
        float row = *imCoordPtr++;
        float col = *imCoordPtr++;
        
        int rowi = (int)(floor(row));
        int coli = (int)(floor(col));

        if (rowi >= 0 && coli >= 0 && rowi < imRows - 1 && coli < imCols - 1)
        {
            float rowf = row - rowi;
            float colf = col - coli;
                
            float* imagePtr = &image.pixelReference(rowi, coli, 0);

            *alphaImPtr++ = 1;

            for (int b = 0; b < nBands; b++)
            {
                float I00 = *imagePtr;
                imagePtr += nBands;
                float I01 = *imagePtr;
                imagePtr += imageRowStride;
                float I11 = *imagePtr;
                imagePtr -= nBands;
                float I10 = *imagePtr;
                imagePtr -= imageRowStride;
                    
                *renderImPtr = 
                    (1.0f - rowf) * (1.0f - colf) * I00 + 
                    rowf  * (1.0f - colf) * I10 + 
                    (1.0f - rowf) * colf * I01 + 
                    rowf * colf * I11;

                renderImPtr++;
                imagePtr++;
            }
        }
        else 
        {
            renderImPtr += nBands;
            alphaImPtr++;
        }

    }// end for i < nPixels
}

// Helper functions for FindLargestRect
void IntegralRows(CImagei &M)
{
    for (int r = 0; r < M.nRows(); r++)
    {
        for (int c = 0; c < (M.nCols() - 1); c++)
        {
            M(r, c + 1, 0) += M(r, c, 0);
        }
    }
}

void IntegralRows(CImagei &Mdst, const CImagei &Msrc)
{
    Mdst = Msrc;
    IntegralRows(Mdst);
}

void IntegralCols(CImagei &M)
{
    for (int c = 0; c < M.nCols(); c++)
    {
        for (int r = 0; r < (M.nRows() - 1); r++)
        {
            M(r + 1, c, 0) += M(r, c, 0);
        }
    }
}

void IntegralCols(CImagei &Mdst, const CImagei &Msrc)
{
    Mdst = Msrc;
    IntegralCols(Mdst);
}

void FirstNonZeroCols(vector<int> &idx, const CImagei &M)
{
    idx.resize(M.nCols());

    for (int c = 0; c < M.nCols(); c++)
    {
        idx[c] = -1;
        bool done = false;
        for (int r = 0; r < M.nRows() && !done; r++)
        {
            if (M(r, c, 0) != 0)
            {
                idx[c] = r;
                done = true;
            }
        }
    }
}

void LastNonZeroCols(vector<int> &idx, const CImagei &M)
{
    idx.resize(M.nCols());
      
    for (int c = 0; c < M.nCols(); c++)
    {
        idx[c] = -1;
        bool done = false;
        for (int r = M.nRows() - 1; r >= 0 && !done; r--)
        {
            if (M(r, c, 0) != 0)
            {
                idx[c] = r;
                done = true;
            }
        }
    }
}

CArrayi FindLargestRect(const CImagei &alphaImage)
{
    // search over all possible interior rectangles  
    CImagei integralRowImage;
    IntegralRows(integralRowImage, alphaImage);

    CImagei integralImage;
    IntegralCols(integralImage, integralRowImage);

    vector<int> firstNonZero, lastNonZero;
    FirstNonZeroCols(firstNonZero, alphaImage);
    LastNonZeroCols(lastNonZero, alphaImage);

    //vector<int> rect(4);
    CArrayi rect(4);

    int nRows2 = alphaImage.nRows() / 2;
    int nCols2 = alphaImage.nCols() / 2;
  
    rect[0] = nRows2;
    rect[1] = nCols2;
    rect[2] = nRows2 + 1;
    rect[3] = nCols2 + 1;
  
    int areaBest = 1;
    
    for (int col0 = 0; col0 < alphaImage.nCols(); col0++)
    {
        for (int col1 = alphaImage.nCols() - 1; col1 > col0; col1--)
        {            
            int row0 = max(firstNonZero[col0], firstNonZero[col1]);
            int row1 = min(lastNonZero[col0],  lastNonZero[col1]);
	  
            if (row0 < 0 || row1 < 0)
                continue;

            int colDiff = col1 - col0;
            int rowDiff = row1 - row0;
            int area = rowDiff * colDiff;
            
            if (area < areaBest)
                continue;
            
            bool done = false;
            while (row0 < row1 && !done)
            {
                int rowIntegral = integralRowImage(row0, col1, 0) - integralRowImage(row0, col0, 0);
                if (rowIntegral == colDiff)
                    done = true;
                else
                    row0++;
            }

            done = false;
            while (row1 > row0 && !done)
            {
                int rowIntegral = integralRowImage(row1, col1, 0) - integralRowImage(row1, col0, 0);
                if (rowIntegral == colDiff)
                    done = true;
                else
                    row1--;
            }

            rowDiff = row1 - row0;
            area = rowDiff * colDiff;
            
            if (area < areaBest)
                continue;

            int areaIntegral = integralImage(row1, col1, 0) + integralImage(row0, col0, 0) - integralImage(row0, col1, 0) - integralImage(row1, col0, 0);
            
            if (area != areaIntegral)
                continue;
            
            rect[0] = row0;
            rect[1] = col0;
            rect[2] = row1;
            rect[3] = col1;
          
            areaBest = area;
        }
    }
  
    return rect;
}
