// ImageFile.h

#pragma once

#include "Array.h"
#include "Image.h"
#include "CrossPlatform.h"
#include "mtiff.h"
#include "mjpeg.h"

// Read/write image using file extension

template <class T> void ReadImage(CImage<T> &im, string filename);
template <class T> void WriteImage(CImage<T> &im, string filename);

// CImageFile -- no cached image

template <class T>
class CImageFile
{
public:
    CImageFile();
    ~CImageFile();

    void Create(string filename, string name, bool writeImages=true); 
    void Create(const CImage<T> &im, string name, bool writeImages=true);

    //void Create(string filename);
    //void Create(const CImage<T> &im);    
    
    void Read(CImage<T> &im); // Read original image
    void ReadMinSize(CImage<T> &im, int minSize); // Read factor 2 resized image whose min dimension is at least minSize
    
    string  OriginalFilename(void) const;
    CArrayi OriginalSize(void) const;
    float   GetScale(const CArrayi &imSize) const;

    void DeleteFiles(void); // Delete all resized image files

    vector<string>  m_filename;
    vector<CArrayi> m_imSize;

    bool m_writeImages;
    bool m_eraseOriginal;
    float m_sigmaBlur;

    string m_name;
};

// CImageFile0 -- cached image

template <class T>
class CImageFile0 : public CImage<T>
{
public:
    CImageFile0();
    ~CImageFile0();

    void Create(string filename);
    void Create(CImage<T> &im);   // NB: this clears the input image im

    void Get(void);               // Get the original image
    void GetMinSize(int minSize); // Get factor 2 resized image whose min dimension is at least minSize
    
    string  OriginalFilename(void) const;
    CArrayi OriginalSize(void) const;
    float   Scale(void) const;

    void DeleteFiles(void); // Delete all resized image files

    vector<string>  m_filename;
    vector<CArrayi> m_imSize;
    
    bool m_eraseOriginal;
    float m_sigmaBlur;
};

// CImageFile -- implementation of template functions

template <class T>
CImageFile<T>::CImageFile()
{ 
    m_writeImages = true;
    m_eraseOriginal = false;
    m_sigmaBlur = 0.5f;
    m_name = TempDir() + DirSep() + "tmp";
}

template <class T>
CImageFile<T>::~CImageFile()
{ 
    DeleteFiles();
}

template <class T>
void CImageFile<T>::Create(const CImage<T> &im, string name, bool writeImages) 
{
    DeleteFiles();

    m_name = name;
    
    string filename0 = m_name + "-0.im";

    //string filename = TempFilename();

    m_filename.push_back(filename0);
    m_imSize.push_back(im.size());
            
    Write(im, filename0);           

    m_eraseOriginal = true;
    m_writeImages = writeImages;
}

template <class T>
void CImageFile<T>::Create(string filename, string name, bool writeImages)
{
    DeleteFiles();    

    m_name = name;

    m_filename.push_back(filename);
    m_writeImages = writeImages;
    m_eraseOriginal = false;
    
    CArrayi dum;
    m_imSize.push_back(dum);
}

/*
template <class T>
void CImageFile<T>::Create(string filename)
{
    string name = TempDir() + DirSep() + "tmp";
    bool writeImages = true;

    Create(filename, name, writeImages);
}

template <class T>
void CImageFile<T>::Create(const CImage<T> &im)
{
    string name = TempDir() + DirSep() + "tmp";
    bool writeImages = true;

    Create(im, name, writeImages);
}
*/

template <class T> 
void CImageFile<T>::Read(CImage<T> &im)
{
    if (m_filename.size()==0)
    {
        cerr << "CImageFile::Read -- no image to read\n";
        return;
    }
    
    //cerr << "CImageFile::Read " << m_filename[0] << endl;
    ReadImage(im, m_filename[0]);
    m_imSize[0] = im.size();
}

template <class T>
void CImageFile<T>::ReadMinSize(CImage<T> &im, int minSize) // Read factor 2 resized image whose min dimension is at least minSize
{    
    int i;
    bool fileExists = false;

    if (m_imSize[0].isEmpty())
        Read(im);

    for (i = 0; i < m_filename.size() && !fileExists; i++)
    {       
        int sz = min(m_imSize[i][0], m_imSize[i][1]);
        
        if (sz/2 < minSize)
            fileExists = true;
    }
   
    int idx = i-1;

    if (fileExists)
    {
        //cerr << "CImageFile::ReadMinSize -- reading " << m_filename[idx] << "\n";
        ReadImage(im, m_filename[idx]);
        return;
    }

    ReadImage(im, m_filename[idx]);
        
    while (min(im.nRows(), im.nCols()) >= 2 * minSize)
    {
        //cerr << "downsample\n";
        ConvolveGaussian(im, m_sigmaBlur);
        DownSample2_3x3(im);
          
        if (m_writeImages)
        {
            string filename = m_name + "-" + NumToString(m_filename.size()) + ".im";
            m_filename.push_back(filename);
            m_imSize.push_back(im.size());        
            //cerr << "CImageFile::ReadMinSize -- creating " << filename << "\n";
            Write(im, filename);        
        }
    }

    return; 
}

template <class T>
string CImageFile<T>::OriginalFilename(void) const
{
    string fn = "";
    
    if (m_filename.size() > 0)
        fn = m_filename[0];

    return fn;
}

template <class T>
CArrayi CImageFile<T>::OriginalSize(void) const
{
    CArrayi is;
    
    if (m_imSize.size() > 0)
        is = m_imSize[0];

    return is;
}

template <class T>
float CImageFile<T>::GetScale(const CArrayi &imSize) const
{
    if (OriginalSize()[0] == 0)
        cerr << "CImageFile::Scale -- orignal size not set\n";
            
    float sc = imSize[0] / (float)OriginalSize()[0];
    
    return sc;      
}

template <class T>
void CImageFile<T>::DeleteFiles(void)
{
    int nFiles = m_filename.size();
    
    if (nFiles > 0)
    {
        if (m_eraseOriginal)
            unlink(m_filename[0].c_str());
        
        for (int i = 1; i < nFiles; i++)
            unlink(m_filename[i].c_str());
    }
    
    m_filename.resize(0);
    m_imSize.resize(0);
}

typedef CImageFile<float>         CImageFilef;
typedef CImageFile<int>           CImageFilei;
typedef CImageFile<unsigned char> CImageFileu;

// CImageFile0 -- implemenation of template functions

// Steal im2 image and file info, clears im2

template <class T> void Steal(CImageFile0<T> &im1, CImageFile0<T> &im2);

// implementation of template functions

template <class T>
CImageFile0<T>::CImageFile0()
{ 
    m_eraseOriginal = false;
    m_sigmaBlur = 0.5f;
}

template <class T>
CImageFile0<T>::~CImageFile0()
{ 
    DeleteFiles();

    /*int nFiles = m_filename.size();
    
      if (nFiles > 0)
      {
      if (m_eraseOriginal)
      unlink(m_filename[0].c_str());
        
      for (int i = 1; i < nFiles; i++)
      unlink(m_filename[i].c_str());
      }
    
      m_filename.resize(0);
      m_imSize.resize(0);*/
}

template <class T>
void CImageFile0<T>::Create(string filename)
{
    this->Clear();
    DeleteFiles();

    CArrayi dum;
    
    m_filename.push_back(filename);
    m_imSize.push_back(dum);
}

template <class T>
void CImageFile0<T>::Create(CImage<T> &im) // NB: this clears the input image im
{
    this->Clear();
    DeleteFiles();

    Steal(*this, im); // steal image data from im
    
    string filename = TempFilename();

    // lin
    //char tmp[] = "/tmp/im-XXXXXX";
    //int fd = mkstemp(tmp);
    //string filename(tmp);
    //close(fd);

    // win
    //char tmp[10];
    //tmpnam_s(tmp, 10);
    //string filename(tmp);
    //filename = "c:\\windows\\temp" + filename + ".im";

    m_filename.push_back(filename);
    m_imSize.push_back(this->size());
            
    Write(*this, filename);           

    m_eraseOriginal=true;
}

template <class T>
void CImageFile0<T>::Get(void)
{
    if (m_filename.size()==0)
    {
        cerr << "CImageFile0::Get -- no image to get\n";
        return;
    }

    bool reload = false;

    if (m_imSize[0].isEmpty())
        reload = true;
    else if (this->nRows() < m_imSize[0][0])
        reload = true;

    if (reload)
    {
        ReadImage(*this, m_filename[0]);
        m_imSize[0] = this->size();
    }
}

template <class T>
void CImageFile0<T>::GetMinSize(int minSize) // Get factor 2 resized image whose min dimension is at least minSize
{    
    int i = 0;
    bool fileExists = false;    

    if (m_imSize[0].isEmpty())
        Get();

    for (i = 0; i < m_filename.size() && !fileExists; i++)
    {       
        int sz = min(m_imSize[i][0], m_imSize[i][1]);
        
        if (sz/2 < minSize)
            fileExists = true;
    }
   
    int idx = i-1;

    if (fileExists)
    {
        if (this->nRows()==m_imSize[idx][0])
        {
            // image already in memory, do nothing
        }
        else
        {
            ReadImage(*this, m_filename[idx]);
        }

        return;
    }

    // resized file does not exist
           
    if (this->nRows()==m_imSize[idx][0])
    {
        // smallest image already in memory, do nothing
    }
    else
    {
        ReadImage(*this, m_filename[idx]);
    }
        
    while (min(this->nRows(),this->nCols()) >= 2 * minSize)
    {
        ConvolveGaussian(*this, m_sigmaBlur);
        DownSample2_3x3(*this);
          
        string filename = TempFilename();

        //char tmp[] = "/tmp/im-XXXXXX";
        //int fd = mkstemp(tmp);
        //string filename(tmp);
        //close(fd);

        m_filename.push_back(filename);
        m_imSize.push_back(this->size());
            
        Write(*this, filename);        
    }
    
}

template <class T>
string CImageFile0<T>::OriginalFilename(void) const
{
    string fn = "";
    
    if (m_filename.size() > 0)
        fn = m_filename[0];

    return fn;
}

template <class T>
CArrayi CImageFile0<T>::OriginalSize(void) const
{
    CArrayi is;
    
    if (m_imSize.size() > 0)
        is = m_imSize[0];

    return is;
}

template <class T>
float CImageFile0<T>::Scale(void) const
{
    float sc = -1.0f;
    
    if (!this->isEmpty())
    {
        int nRows = this->nRows();
        int nRows0 = OriginalSize()[0];
        sc = nRows/(float)nRows0;
    }

    return sc;      
}

template <class T>
void CImageFile0<T>::DeleteFiles(void)
{
    int nFiles = m_filename.size();
    
    if (nFiles > 0)
    {
        if (m_eraseOriginal)
            unlink(m_filename[0].c_str());
        
        for (int i = 1; i < nFiles; i++)
            unlink(m_filename[i].c_str());
    }
    
    m_filename.resize(0);
    m_imSize.resize(0);
}

template <class T>
void Steal(CImageFile0<T> &im1, CImageFile0<T> &im2)
{
    // steal image data
    CImage<T> *im1ptr = (CImage<T> *)(&im1);
    CImage<T> *im2ptr = (CImage<T> *)(&im2);    
    Steal(*im1ptr, *im2ptr);

    // copy metadata    
    im1.m_filename       = im2.m_filename;
    im1.m_fileDescriptor = im2.m_fileDescriptor;
    im1.m_imSize         = im2.m_imSize;    
    im1.m_eraseOriginal  = im2.m_eraseOriginal;
    im1.m_sigmaBlur      = im2.m_sigmaBlur;
}

typedef CImageFile0<float>         CImageFile0f;
typedef CImageFile0<int>           CImageFile0i;
typedef CImageFile0<unsigned char> CImageFile0u;

// Read/write image using file extension

template <class T>
void ReadImage(CImage<T> &im, string filename)
{
    string ext = GetFileExtension(filename);

    if (ext=="jpg")
    {
        ReadJPEG(im, filename);
    }
    else if (ext=="tif"||ext=="tiff")
    {
        ReadTIFF(im, filename);        
    }
    else if (ext=="ppm")
    {
        ReadPPM(im, filename.c_str());
    }
    //else if (ext=="im"||ext=="")
    else 
    {
        // assume CImage
        Read(im, filename);
    }
}

template <class T>
void WriteImage(CImage<T> &im, string filename)
{
    string ext = GetFileExtension(filename);

    if (ext=="jpg")
    {
        WriteJPEG(im, filename.c_str());
    }
    else if (ext=="tif"||ext=="tiff")
    {
        WriteTIFF(im, filename);        
    }
    else if (ext=="ppm")
    {
        WritePPM(im, filename.c_str());
    }
    //else if (ext=="im"||ext=="")
    else
    {
        // assume CImage
        Write(im, filename);
    }
}
