function xout = load_image(filename, pathname, input_type, cfa_ind, nir_info, cfa, premul)
%*************************************************************************%
% Author: Dominic Rfenacht, December 2011
% Last revision: 21.08.2012
% Purpose:
% This function is used to load image(s) that contain both visible and NIR
% information. The input can either come from a two-shot solution, or
% (hopefully soon) from a single VIS/NIR sensor.
%*************************************************************************%


%Load the image to be processed
switch input_type
    case 'vis'
        %Convert the visible RAW image to 16 bit linear, mosaiced TIFF.
        % -t 0 : Do not flip the image. This way, the CFA is always in the same
        % order
        % - o 0: Use the camera colors
        
        status = unix(['dcraw -o 0 -d -t 0 -T -4 -r ' num2str(premul(1)) ' ' num2str(premul(2)) ' ' num2str(premul(3)) ' ' num2str(premul(2)) ' "' pathname filename '"']);
        %status = unix(['dcraw -o 0 -D -t 0 -T -4 "' pathname filename '"']);
        if(status ~= 0)
            error('Conversion failed. Make sure that DCRAW is installed!')
        end
        %Get the filename without extension
        filebase = filename(1:strfind(filename, '.')-1);
        %Load the RAW (mosaic) image containing the visible information
        image = imread([pathname filebase '.tiff']);
        [rows, cols, ~] = size(image);
        image = image(1:end-mod(rows, 2), 1:end-mod(cols, 2), :);
        %{
        %----- Alternative Way to use with the -D option instead -d
        [rows, cols, c] = size(image);
        
        image = double(image);
        
        %Gaussian blur the image to get rid of outliers
        sigma = 5;
        
        toto = zeros(rows/2, cols/2, c);
        f = gaussFilter(sigma);
        
        toto(:,:,1) = imfilter(image(cfa_ind(1,1):2:end, cfa_ind(1,2):2:end), f, 'symmetric', 'same', 'conv');
        toto(:,:,2) = imfilter(image(cfa_ind(2,1):2:end, cfa_ind(2,2):2:end), f, 'symmetric', 'same', 'conv');
        toto(:,:,3) = imfilter(image(cfa_ind(3,1):2:end, cfa_ind(3,2):2:end), f, 'symmetric', 'same', 'conv');
        image = double(image);
        
        MAX =  max(max(toto(:)));
        MIN = min(min(toto(:)));
        
        image(image > MAX) = MAX;
        image(image < MIN) = MIN;
        
        image = (image - MIN)./(MAX - MIN);
        
        red = image(cfa_ind(1,1):2:end, cfa_ind(1,2):2:end);
        green1 = image(cfa_ind(2,1):2:end, cfa_ind(2,2):2:end);
        green2 = image(cfa_ind(4,1):2:end, cfa_ind(4,2):2:end);
        blue = image(cfa_ind(3,1):2:end, cfa_ind(3,2):2:end);

        %This is to avoid that highlights are pink
        red(red > 1./premul(1)) = 1./premul(1);
        green1(green1 > 1./premul(2)) = 1./premul(2);
        green2(green2 > 1./premul(2)) = 1./premul(2);
        blue(blue > 1./premul(3)) = 1./premul(3);
        
        %Apply multipliers
        red = red.*premul(1);
        green1 = green1.*premul(2);
        green2 = green2.*premul(2);
        blue = blue.*premul(3);
        
        image(cfa_ind(1,1):2:end, cfa_ind(1,2):2:end) = red;
        image(cfa_ind(2,1):2:end, cfa_ind(2,2):2:end) = green1;
        image(cfa_ind(3,1):2:end, cfa_ind(3,2):2:end) = blue;
        image(cfa_ind(4,1):2:end, cfa_ind(4,2):2:end) = green2;
        
        image = uint16(image.*65535);
        %----- END of Alternative Way to use with the -D option instead -d
        %}
        %{
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%QUALCOMM
        % Open file
        id = fopen([pathname filename], 'r');

        % Read in the data.
        image = fread(id, [4096, 1152], 'uint8')';
        
        %Linearize the image
        %image = image./max(max(image));
        %image = image.^(2.4);
        %image = image./max(max(image))*65535;
        %Put between [0, 65535] (for AMAZE demosaicing)
        %image = image./max(max(image))*65535;
        % Close file
        fclose(id);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%END%QUALCOMM
        %}
        %Extract the color channels.
        [rows, cols, ~] = size(image);
        image = image(1:end-mod(rows, 2), 1:end-mod(cols, 2), :);
        [rows, cols, ~] = size(image);
        xout = double(zeros(rows, cols, 3));
        
        %Load the color information according to the CFA arrangement
        count = 1;
        for i = 1:2
            for j = 1:2
                xout(i:2:end, j:2:end, getIndexFromString(cfa(count))) = image(i:2:end, j:2:end);
                count = count + 1;
            end
        end
        
        [m,n,~] = size(xout);
        %Make sure the dimensions of the image divide the size of the CFA
        xout = xout(1:end-mod(m, 2), 1:end-mod(n, 2), :);
        
    case 'nir'
        %Convert the NIR RAW image to 16 bit linear, mosaiced TIFF.
        % -t 0 : Do not flip the image. This way, the CFA is always in the same
        % order
        % - o: Use the camera colors
        status = unix(['dcraw -o 0 -d -t 0 -T -4 -r ' num2str(premul(1)) ' ' num2str(premul(2)) ' ' num2str(premul(3)) ' ' num2str(premul(2)) ' "' pathname filename '"']);
        if(status ~= 0)
            error('Conversion failed. Make sure that DCRAW is installed!')
        end
        
        %Get the filename without extension
        filebase = filename(1:strfind(filename, '.')-1);
        %Load the RAW (mosaic) image containing the visible information
        image = imread([pathname filebase '.tiff']);
        
        %Extract the color and nir channels.
        [rows, cols, ~] = size(image);
        image = image(1:end-mod(rows, 2), 1:end-mod(cols, 2), :);
        [rows, cols, ~] = size(image);
        xout = double(zeros(rows, cols));
        
        %Load the NIR information according to the CFA arrangement
        count = 1;
        for i = 1:2
            for j = 1:2
                if(strcmp(nir_info, cfa(count)) || strcmp(nir_info, 'all'))
                    xout(i:2:end, j:2:end) = image(i:2:end, j:2:end);
                end
                count = count + 1;
            end
        end
        
        [m,n,~] = size(xout);
        %Make sure the dimensions of the image divide the size of the CFA
        xout = xout(1:end-mod(m, 2), 1:end-mod(n, 2), :);
        
    case 'visnir_vis'
        %Convert the RAW image to 16 bit linear, mosaiced TIFF.
        % -t 0 : Do not flip the image. This way, the CFA is always in the same
        % order
        % - o: Use the camera colors
        status = unix(['dcraw -o 0 -d -t 0 -T -4 -r ' num2str(premul(1)) ' ' num2str(premul(2)) ' ' num2str(premul(3)) ' ' num2str(premul(2)) ' "' pathname filename '"']);
        
        if(status ~= 0)
            error('Conversion failed. Make sure that DCRAW is installed!')
        end
        %Get the filename without extension
        filebase = filename(1:strfind(filename, '.')-1);
        %Load the RAW (mosaic) image containing the visible information
        image = imread(['./images/visnir/' filebase '.tiff']);
        
        %Extract the color and nir channels.
        [rows, cols, ~] = size(image);
        image = image(1:end-mod(rows, 2), 1:end-mod(cols, 2), :);
        [rows, cols, ~] = size(image);
        xout = double(zeros(rows, cols, 3));
        
        %Load the color information according to the CFA arrangement
        count = 1;
        for i = 1:2
            for j = 1:2
                xout(i:2:end, j:2:end, getIndexFromString(cfa(count))) = image(i:2:end, j:2:end);
                count = count + 1;
            end
        end
        [m,n,~] = size(xout);
        %Make sure the dimensions of the image divide the size of the CFA
        xout = xout(1:end-mod(m, 2), 1:end-mod(n, 2), :);
        
    case 'visnir_nir'
        %Convert the RAW image to 16 bit linear, mosaiced TIFF.
        % -t 0 : Do not flip the image. This way, the CFA is always in the same
        % order
        % - o: Use the camera colors
        status = unix(['dcraw -o 0 -d -t 0 -T -4 -r ' num2str(premul(1)) ' ' num2str(premul(2)) ' ' num2str(premul(3)) ' ' num2str(premul(2)) ' "' pathname filename '"']);
        
        if(status ~= 0)
            error('Conversion failed. Make sure that DCRAW is installed!')
        end
        %Get the filename without extension
        filebase = filename(1:strfind(filename, '.')-1);
        %Load the RAW (mosaic) image containing the visible information
        image = imread(['./images/visnir/' filebase '.tiff']);
        
        %Extract the color and nir channels.
        [rows, cols, ~] = size(image);
        image = image(1:end-mod(rows, 2), 1:end-mod(cols, 2), :);
        [rows, cols, ~] = size(image);
        xout = double(zeros(rows, cols));
        
        %Load the NIR information according to the CFA arrangement
        count = 1;
        for i = 1:2
            for j = 1:2
                if(strcmp(nir_info, cfa(count)) || strcmp(nir_info, 'all'))
                    xout(i:2:end, j:2:end) = image(i:2:end, j:2:end);
                end
                count = count + 1;
            end
        end
        
        [m,n,~] = size(xout);
        %Make sure the dimensions of the image divide the size of the CFA
        xout = xout(1:end-mod(m, 2), 1:end-mod(n, 2), :);
        
    otherwise
        error('The input type you specified is not accepted.');
end