function camspecs = plot_spectral_response(filenames, camera_brand, camera_make, filter_type, spectrum, save_dir)
%*************************************************************************%
% Author: Dominic Rfenacht, December 2011
% Purpose: 
% This function is used to plot the spectral responses based on the
% txt-files generated by the camSPECS tool.
% Example usage:
% plot_spectral_response({'specs_measurement1.txt', 'specs_measurement2.txt', 'specs_measurement3.txt'}, 'Canon', 'EOS 5D Mark II', 'UV', 'visible', 'D:/my_measurements/')
%*************************************************************************%
%Input: - filenames: cell array (see example usage above) containing the names of the txt-files
%         generate by the camSPECS tool. Note that these txt-files need to be in the
%         same folder as the plot-function.
%       - camera_brand: Brand name of the camera
%       - camera_make: Make of the camera
%       - filter_type: Type of the filter used during the measurements
%       - Spectrum: Indicates the spectrum that will be plotted. Options
%         are:
%           1) 'full': From 380-1100 nm
%           2) 'visible' : From 380-750 nm
%           3) 'nir': From 750-1100 nm
%       - save_dir: Directory where the plot will be stored. (OPTIONAL)
%Output: - camspecs: The average spectral responses (standardized output)
%        - (Plot of the spectral response)
%*************************************************************************%

if(nargin == 5)
    %Set the base where the plot(s) will be stored
    save_dir = 'D:/Spectral_Sensitivities/Results/Plots/';
end

if(strcmp(filter_type, 'none'))
    fprintf(['Plotting the spectral response function of a ' camera_brand ' ' camera_make ' without filter... '])
else
    fprintf(['Plotting the spectral response function of a ' camera_brand ' ' camera_make ' with filter ' filter_type '...'])
end

num_measurements = length(filenames);
%Read the relevant information
t = textread(filenames{1}, '%s');
%Get the position where the "interesting" information starts
start_index = find(strcmp(t, '380'));
%Extract the relevant information
t = t(start_index:end);

if(length(t) == 648)
    num_cols = 12;
    monochrome = 0;
elseif(length(t) == 324)
    num_cols = 6;
    monochrome = 1;
else
    error('There is something wrong with the text file...')
end

specs = zeros(54, num_cols, num_measurements);

for i = 1:num_measurements
    %Read the relevant information
    t = textread(filenames{i}, '%s');
    %Get the position where the "interesting" information starts
    start_index = find(strcmp(t, '380'));
    %Extract the relevant information
    t = t(start_index:end);
    %Put into matrix form
    specs(:,:,i) = cell2num(reshape(t, num_cols, 54)');
    
end

%Define the spectrum that needs to be plotted
if(strcmp(spectrum, 'full'))
    sp = 1:54;
elseif(strcmp(spectrum, 'visible'))
    sp = 1:38;
elseif(strcmp(spectrum, 'nir'))
    sp = 38:54;
else
    error('The spectrum specified cannot be displayed. Valid options are: full, visible and nir.')
end

%Average the measurements
avg_specs = mean(specs, 3);

if(monochrome == 0)
    %Standard deviations
    std_specs = std(specs, 0, 3);
    std_specs = std_specs(:, 2:4);
    
    %Plot the spectral sensitivity curve
    h = plot(avg_specs(sp,1), avg_specs(sp,4), avg_specs(sp,1), avg_specs(sp,3), avg_specs(sp,1), avg_specs(sp,2));
    set(h, 'LineWidth', 1.5)
    if(strcmp(filter_type, 'none'))
        title(['Spectral sensitivities for ' camera_brand ' ' camera_make  ' without filter (average of ' num2str(num_measurements) ' measurements)']);
    else
        title(['Spectral sensitivities for ' camera_brand ' ' camera_make  ' with filter ' filter_type ' (average of ' num2str(num_measurements) ' measurements)']);
    end
    
    xlabel('Wavelength \lambda [nm]')
    ylabel('Spectral Response')
    legend('Blue', 'Green', 'Red')
    axis([specs(sp(1), 1, 1) specs(sp(end), 1, 1) 0 1]);
    fprintf('Done! \n')
    disp(['Maximum standard deviation is ' num2str(max(std_specs(:))) '...'])
    
    %Save the plot
    savepath = [save_dir camera_brand '/' camera_make '/'];
    if(~exist(savepath, 'dir'))
        mkdir(savepath)
    end
    saveas(gcf, [savepath camera_make '_' spectrum '_' filter_type '.png'])
    fprintf(['Plot saved to ' savepath camera_make '_' spectrum '_' filter_type '.png \n'])
    
    %Write text-file in readable format
    for i = 1:num_measurements
        filename = [savepath camera_make '_' spectrum '_' filter_type '_' num2str(i) '.csv'];
        fid = fopen(filename, 'w');
        
        %Write header
        fprintf(fid, 'Lambda;R_SR;G_SR;B_SR;R_REF;G_REF;B_REF;EV;R_RAW;G_RAW;B_RAW;Filter calibration\n');
        %Close the file
        fclose(fid);
        
        %Write data
        dlmwrite(filename, avg_specs, '-append', 'delimiter', ';');
        
    end
    
else
    warning('Attention, only one channel information is available! You should change the settings of the camSPECS tool and manually enter the CFA arrangement, since it could not be automatically detected. This happens if only NIR information is available...')
    %Standard deviations
    std_specs = std(specs, 0, 3);
    std_specs = std_specs(:, 2);
    
    %Plot the spectral sensitivity curve
    plot(avg_specs(sp,1), avg_specs(sp,2));
    if(strcmp(filter_type, 'none'))
        title(['Spectral sensitivities for ' camera_brand ' ' camera_make  ' without filter (average of ' num2str(num_measurements) ' measurements)']);
    else
        title(['Spectral sensitivities for ' camera_brand ' ' camera_make  ' with filter ' filter_type ' (average of ' num2str(num_measurements) ' measurements)']);
    end
    
    xlabel('Wavelength \lambda [nm]')
    ylabel('Spectral Response')
    legend('NIR')
    axis([specs(sp(1), 1, 1) specs(sp(end), 1, 1) 0 1]);
    fprintf('Done! \n')
    disp(['Maximum standard deviation is ' num2str(max(std_specs(:))) '...'])
    
    %Save the plot
    savepath = [save_dir camera_brand '/' camera_make '/'];
    if(~exist(savepath, 'dir'))
        mkdir(savepath)
    end
    saveas(gcf, [savepath camera_make '_' spectrum '_' filter_type '.png'])
    fprintf(['Plot saved to ' savepath camera_make '_' spectrum '_' filter_type '.png \n'])
    
    %Write text-file in readable format
    for i = 1:num_measurements
        filename = [savepath camera_make '_' spectrum '_' filter_type '_' num2str(i) '.csv'];
        fid = fopen(filename, 'w');
        
        %Write header
        fprintf(fid, 'Lambda;SR;REF;EV;RAW;Filter calibration\n');
        %Close the file
        fclose(fid);
        
        %Write data
        dlmwrite(filename, avg_specs, '-append', 'delimiter', ';');
        
    end  
end

%camspecs = avg_specs(3:33, [2 3 4]);
camspecs = avg_specs(:, [1 2 3 4]);